"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

/*
  Copyright 2018 Google LLC

  Use of this source code is governed by an MIT-style
  license that can be found in the LICENSE file or at
  https://opensource.org/licenses/MIT.
*/
const _require = require('webpack-sources'),
      RawSource = _require.RawSource;

const bundle = require('workbox-build/build/lib/bundle');

const populateSWTemplate = require('workbox-build/build/lib/populate-sw-template');

const validate = require('workbox-build/build/entry-points/options/validate');

const webpackGenerateSWSchema = require('workbox-build/build/entry-points/options/webpack-generate-sw-schema');

const getManifestEntriesFromCompilation = require('./lib/get-manifest-entries-from-compilation');

const relativeToOutputPath = require('./lib/relative-to-output-path');
/**
 * This class supports creating a new, ready-to-use service worker file as
 * part of the webpack compilation process.
 *
 * Use an instance of `GenerateSW` in the
 * [`plugins` array](https://webpack.js.org/concepts/plugins/#usage) of a
 * webpack config.
 *
 * @module workbox-webpack-plugin
 */


class GenerateSW {
  /**
   * Creates an instance of GenerateSW.
   *
   * @param {Object} [config] See the
   * [configuration guide](https://developers.google.com/web/tools/workbox/modules/workbox-webpack-plugin#configuration)
   * for all supported options and defaults.
   */
  constructor(config = {}) {
    this.config = config;
  }
  /**
   * @param {Object} [compiler] default compiler object passed from webpack
   *
   * @private
   */


  propagateWebpackConfig(compiler) {
    // Because this.config is listed last, properties that are already set
    // there take precedence over derived properties from the compiler.
    this.config = Object.assign({
      mode: compiler.options.mode,
      sourcemap: Boolean(compiler.options.devtool)
    }, this.config);
  }
  /**
   * @param {Object} [compiler] default compiler object passed from webpack
   *
   * @private
   */


  apply(compiler) {
    this.propagateWebpackConfig(compiler);
    compiler.hooks.emit.tapPromise(this.constructor.name, compilation => this.handleEmit(compilation).catch(error => compilation.errors.push(error)));
  }
  /**
   * @param {Object} compilation The webpack compilation.
   *
   * @private
   */


  handleEmit(compilation) {
    var _this = this;

    return (0, _asyncToGenerator2.default)(function* () {
      try {
        _this.config = validate(_this.config, webpackGenerateSWSchema);
      } catch (error) {
        throw new Error(`Please check your ${_this.constructor.name} plugin ` + `configuration:\n${error.message}`);
      }

      _this.config.manifestEntries = getManifestEntriesFromCompilation(compilation, _this.config);
      const unbundledCode = populateSWTemplate(_this.config);
      const files = yield bundle({
        babelPresetEnvTargets: _this.config.babelPresetEnvTargets,
        inlineWorkboxRuntime: _this.config.inlineWorkboxRuntime,
        mode: _this.config.mode,
        sourcemap: _this.config.sourcemap,
        swDest: relativeToOutputPath(compilation, _this.config.swDest),
        unbundledCode
      });
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = files[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          const file = _step.value;
          compilation.assets[file.name] = new RawSource(file.contents);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    })();
  }

}

module.exports = GenerateSW;